unit Unit1;

interface

uses
  {$ifdef MSWINDOWS}
  Windows,
  Messages,
  Graphics,
  {$endif}
  Classes, SysUtils, Forms, Controls, Dialogs, StdCtrls,
  SynCommons,
  mORMot,
  SampleData,
  RVStyle, RVScroll,
  RichView, RVEdit, RVLinear;

type

  { TForm1 }

  TForm1 = class(TForm)
    AddButton: TButton;
    Label1: TLabel;
    Label2: TLabel;
    QuitButton: TButton;
    FindButton: TButton;
    NameEdit: TEdit;
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Label3: TLabel;
    Memo1: TMemo;
    GenMessageButton: TButton;
    procedure AddButtonClick(Sender: TObject);
    procedure FindButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure QuitButtonClick(Sender: TObject);
    procedure GenMessageButtonClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    Database: TSQLRest;
    Model: TSQLModel;
  end;

var
  Form1: TForm1;

implementation

{$ifdef FPC}
{$R *.lfm}
{$else}
{$R *.dfm}
{$endif}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  Model := CreateSampleModel; // from SampleData unit
end;

function GetDocument(rv: TCustomRichView): TSQLRawBlob;
var
  Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  rv.SaveRVFToStream(Stream, False);
  SetLength(Result, Stream.Size);
  Stream.Position := 0;
  Stream.ReadBuffer(Pointer(Result)^, Stream.Size);
  Stream.Free;
end;

procedure SetDocument(const blob: TSQLRawBlob; rv: TCustomRichView);
var
  Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  Stream.WriteBuffer(Pointer(blob)^, Length(blob));
  Stream.Position := 0;
  rv.LoadRVFFromStream(Stream);
  rv.Format;
  Stream.Free;
end;

function GetPlainText(rv: TCustomRichView): RawUTF8;
begin
  Result := UTF8Encode(RVGetTextRange(rv, 0, RVGetTextLength(rv)));
end;

procedure TForm1.AddButtonClick(Sender: TObject);
var Rec: TSQLSampleRecord;
begin
  if NameEdit.Text = '' then
  begin
    Application.MessageBox('Please fill Name', 'Error');
    exit;
  end;

  Rec := TSQLSampleRecord.Create;
  try
    // we use explicit StringToUTF8() for conversion below
    // a real application should use TLanguageFile.StringToUTF8() in mORMoti18n
    Rec.Name := StringToUTF8(NameEdit.Text);
    Rec.Document := GetDocument(RichViewEdit1);
    Rec.PlainText := GetPlainText(RichViewEdit1);
    if Database.AddWithBlobs(Rec,true)=0 then
      ShowMessage('Error adding the data') else begin
      NameEdit.Text := '';
      RichViewEdit1.Clear;
      RichViewEdit1.DeleteUnusedStyles(True, True, True);
      RichViewEdit1.Format;
      Memo1.Lines.Clear;
      NameEdit.SetFocus;
    end;
  finally
    Rec.Free;
  end;
end;


procedure TForm1.FindButtonClick(Sender: TObject);
var Rec: TSQLSampleRecord;
begin
  Rec := TSQLSampleRecord.Create(Database,'Name=?',[StringToUTF8(NameEdit.Text)]);
  try
    if Rec.ID=0 then
    begin
      Application.MessageBox('Not found!', 'Not Found', MB_OK)
    end
    else
    begin
      Database.RetrieveBlobFields(Rec);
      SetDocument(Rec.Document, RichViewEdit1);
      Memo1.Text := UTF8ToString(Rec.PlainText);
    end;
  finally
    Rec.Free;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Database.Free;
  Model.Free;
end;

procedure TForm1.QuitButtonClick(Sender: TObject);
begin
  Close;
end;

{------------------------------------------------------------------------------}
{ Sample message generation }

// Returing the index of item in RVStyle.TextStyles[]. This item has
// the same properties as RVStyle.TextStyles[0], but the specified Color.
// If not exists, a new such item is added and returned.
function GetColoredTextStyle(RVStyle: TRVStyle; Color: TColor): Integer;
var
  TextStyle: TFontInfo;
begin
  TextStyle := TFontInfo.Create(nil);
  TextStyle.Assign(RVStyle.TextStyles[0]);
  TextStyle.Color := Color;
  Result := RVStyle.FindTextStyle(TextStyle);
  TextStyle.Free;
end;

// Generating a sample document in RichViewEdit1
procedure TForm1.GenMessageButtonClick(Sender: TObject);
var
  Color: TColor;
  i: Integer;
  bmp: TBitmap;
begin
  RichViewEdit1.Clear;
  RichViewEdit1.DeleteUnusedStyles(True, True, True);
  for I := 0 to 5 do
  begin
    Color := Random(clWhite);
    RichViewEdit1.AddNL('This line has color ' + IntToStr(Color) +' ',
     GetColoredTextStyle(RVStyle1, Color), 0);
    bmp := TBitmap.Create;
    bmp.Width := 16;
    bmp.Height := 16;
    bmp.Canvas.Brush.Color := Color;
    bmp.Canvas.Rectangle(0, 0, bmp.Width, bmp.Height);
    RichViewEdit1.AddPicture('', bmp);
  end;
  RichViewEdit1.Format;
end;



end.

